@@ -28,7 +28,7 @@
 
 #include "x265.h"
 
-using namespace x265;
+using namespace X265_NS;
 
 ShortYuv::ShortYuv()
 {
