@@ -28,7 +28,7 @@
 #include "bitstream.h"
 #include "slice.h"
 
-namespace x265 {
+namespace X265_NS {
 // private namespace
 
 class SEI : public SyntaxElementWriter
